<?php
namespace HCM\FPA\Calculos\Geral;

use HCM\Util\TkDI;
use HCM\Util\DateUtil;
use HCM\Util\MsgCalculo;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;
use HCM\Geral\Entity\EnderecoparcEntity;
use HCM\Geral\Entity\ComunicaparcEntity;
use HCM\Geral\Entity\BairroEntity;

class Estruturacao {

    const ENDERECO_PRINCIPAL = 'PRINCIPAL';
    const FORMA_COMUNICACAO_TEL_PRINCIPAL = '01';
    const FORMA_COMUNICACAO_EMAIL = '05';

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_ultimoDiaCompetencia;
    private $_dtAux;
    private $_estrutura;
    private $_estruturaTomador;
    private $_hierarquiaOrg;
    private $_hierarquiaOrgTomador;
    private $_tipoHierarquia;
    private $_tipoHierarquiaTomador;
    private $_movimentacao;
    private $_movimentacaoVinculoResponsavel;
    private $_movimentacoesVinculoTomador;

    private $_areaRisco;
    private $_vinculoAreaRisco;
    private $_estruturaAreaRisco;

    private $_enderecoParc;
    private $_comunicaParc;
    private $_municipio;
    private $_bairro;
    private $_listaDDD = array(
        "SP" => '011',
        "RJ" => '022',
        "ES" => '027',
        "MG" => '031',
        "PR" => '041',
        "SC" => '048',
        "RS" => '051',
        "DF" => '061',
        "GO" => '062',
        "TO" => '063',
        "MT" => '065',
        "MS" => '067',
        "AC" => '068',
        "RO" => '069',
        "BA" => '071',
        "SE" => '079',
        "PE" => '081',
        "AL" => '082',
        "PB" => '083',
        "RN" => '084',
        "CE" => '085',
        "PI" => '086',
        "PA" => '091',
        "AM" => '092',
        "RR" => '095',
        "AP" => '096',
        "MA" => '098',
    );

    public function __construct($entityManager, $hcmParameters){

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia());
        $this->_ultimoDiaCompetencia = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        $this->_estrutura = new EstruturamEntity($entityManager, $hcmParameters);
        $this->_estruturaTomador = new EstruturamEntity($entityManager, $hcmParameters);
        $this->_hierarquiaOrg = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_hierarquiaOrgTomador = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_tipoHierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_tipoHierarquiaTomador = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_movimentacao = new GpemovimentacaoEntity($entityManager, $hcmParameters);

/*        $this->_areaRisco = new SstAreariscomEntity($entityManager, $hcmParameters);
        $this->_vinculoAreaRisco = new SstVincareariscoEntity($entityManager, $hcmParameters);
        $this->_estruturaAreaRisco = new SstEstrutareariscoEntity($entityManager, $hcmParameters);
*/
        $this->_enderecoParc = new EnderecoparcEntity($entityManager, $hcmParameters);
        //$this->_municipio = new MunicipioEntity($entityManager, $hcmParameters);
        $this->_bairro = new BairroEntity($entityManager, $hcmParameters);
        $this->_comunicaParc = new ComunicaparcEntity($entityManager, $hcmParameters);

    }

    public function getAreaRisco(){
        return $this->_areaRisco;
    }

    public function carregaHierarquias($nrTpHierarquiaOrg){
        $this->_hierarquiaOrg->carregaHierarquiasPorTipo($this->_nrOrg, $nrTpHierarquiaOrg);
    }
    
    public function carregaHierarquiasTomador($nrTpHierarquiaOrgTomador){
        $this->_hierarquiaOrgTomador->carregaHierarquiasPorTipo($this->_nrOrg, $nrTpHierarquiaOrgTomador);
    }

    public function carregaTipoHierarquia($nrTpHierarquiaOrg){
        $this->_tipoHierarquia->carregaTipoHierarquia($this->_nrOrgPadrao, $this->_nrOrg, $nrTpHierarquiaOrg);
    }

    public function carregaTipoHierarquiaTomador($nrTpHierarquiaOrgTomador){
        $this->_tipoHierarquiaTomador->carregaTipoHierarquia($this->_nrOrgPadrao, $this->_nrOrg, $nrTpHierarquiaOrgTomador);
    }

    public function getMovimentacaoVinculoResponsavel(){
        return $this->_movimentacaoVinculoResponsavel;
    }
    
    public function getMovimentacaoVinculoTomador($posicao){
        return $this->_movimentacoesVinculoTomador[$posicao];
    }
    
    public function retornaEstrutura($nrEstrutura) {
        return $this->_estrutura->retornaEstrutura($nrEstrutura);
    }

    private function _verificaPeriodoValidoMes($data){
        $this->_dtAux = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        $dtInicio = $this->_mesCompetencia->format('d/m/Y');
        $dtFim = $this->_dtAux->format('d/m/Y');

        if (empty($data) === false) {
            $existe = $this->_verificaDataPeriodo($data, $dtInicio, $dtFim);
        } else {
            $existe = true;
        }
        
        return $existe;
    }

    private function _verificaDataPeriodo($data, $dtInicio, $dtFim){
        $this->_dtAux->set($data);
        if ($this->_dtAux->isLater($dtInicio) || $this->_dtAux->equals($dtInicio)
           && $this->_dtAux->isEarlier($dtFim) || $this->_dtAux->equals($dtFim) ) {
            return true;
        } else {
            return false;
        }
    }
    
    public function incluiMovimentacaoVinculo(
        $nrTipoTransf, $nrVinculo, $nrEstrutura, $dtIniMoviment, $nrTipoEstrutura, $nrTpMovTransf, $dtRetroativa,
        $idAtualizacao, $dtFimMovimento
    ){
        $mov = $this->_movimentacao->incluiMovimentacao(
            $this->_nrOrg, $nrTipoTransf, $nrVinculo, $nrEstrutura, $dtIniMoviment, $nrTipoEstrutura, $nrTpMovTransf, $dtRetroativa,
            $idAtualizacao, $dtFimMovimento
        );
        return $mov;
    }

    public function verificaExistenciaMovimentacao($nrTpMovTransferencia, $dtIniMovimento, $dtRetroativa){
        $verificaMovimentacao = $this->_movimentacao->verificaMovPorTipoTransferencia(
            $nrTpMovTransferencia, $dtIniMovimento, $dtRetroativa
        );

        return $verificaMovimentacao;
    }
    
    public function getMovimentacoesVinculo($nrVinculo){
        $this->_movimentacao->carregaMovimentacoes($this->_nrOrg, $nrVinculo, $this->_ultimoDiaCompetencia);

        return $this->_movimentacao->getItems();
    }

    public function getMovimentacoesVinculoPorTipo($nrVinculo, $tipo){
        $tipoEstrutura = FPA::getTipoEstruturaLegalOrg();
        $this->_movimentacao->carregaMovimentacoesPorTipo(
            $this->_nrOrg, $nrVinculo, $tipo, $this->_ultimoDiaCompetencia, $tipoEstrutura
        );

        return $this->_movimentacao->getItems();
    }

    public function getMovimentacaoVinculoPorTipo($nrVinculo, $tipo){
        $tipoEstrutura = FPA::getTipoEstruturaLegalOrg();
        $this->_movimentacao->carregaMovimentacoesPorTipo(
            $this->_nrOrg, $nrVinculo, $tipo, $this->_ultimoDiaCompetencia, $tipoEstrutura
        );

        return current($this->_movimentacao->getItems());
    }
    
    /*Específico para o holerite*/
    public function getMovimentacaoVinculoPorTipoHolerite($nrVinculo, $tipo){
        $tipoEstrutura = FPA::getTipoEstruturaLegalOrg();
        $this->_movimentacao->carregaMovimentacoesPorTipoHolerite(
            $this->_nrOrg, $nrVinculo, $tipo, $this->_ultimoDiaCompetencia, $tipoEstrutura
        );

        return current($this->_movimentacao->getItems());
    }

    public function getEstruturas($nrVinculom, $nrTipoHierarquia){

        $paramsTipoHierarquia = array($this->_nrOrgPadrao, $this->_nrOrg, $nrTipoHierarquia);
        $tipoHierarquia = TipohierarqorgEntity::getEntityByWhere(
            '(NRORG = ? OR NRORG = ?) AND NRTIPOHIERARQORG = ?', $paramsTipoHierarquia
        );
        
        $this->_movimentacao->carregaMovimentacoesPorHierarquia($this->_nrOrg, $nrVinculom,
            $tipoHierarquia->getNrtipoestrutrel(), $this->_mesCompetencia, $this->_ultimoDiaCompetencia);

        foreach($this->_movimentacao->getItems() as $item){
            $this->_estrutura->carregaEstrutura($item->getNrestruturam());
            $estrutura = $this->_estrutura->getEstrutura();
            if($estrutura){
                return $estrutura;
            }
        }
        return null;

    }

    public function getEstruturaSuperior($movimentacao){

        $hierarquias = $this->_hierarquiaOrg->getItems();
        $tpHierarquia = $this->_tipoHierarquia->getItem(0)[0];
        
        if ((int)$movimentacao->getNrtipoestrutura() === (int)$tpHierarquia->getNrtipoestrutrel()) {
            $nrEstruturaSup = $this->_determinaEstruturaSuperior(
                $hierarquias, $movimentacao->getNrestruturam(), $tpHierarquia->getIdautohierarq()
            );
            $this->_estrutura->carregaEstrutura($nrEstruturaSup);
            $estrutura = $this->_estrutura->getEstrutura();

            return $estrutura;
        }
    }
    
    public function getEstruturaSuperiorTomador($movimentacao){

        $hierarquiasTomador = $this->_hierarquiaOrgTomador->getItems();
        $tpHierarquiaTomador = $this->_tipoHierarquiaTomador->getItem(0)[0];
        if ((int)$movimentacao->getNrtipoestrutura() === (int)$tpHierarquiaTomador->getNrtipoestrutrel()) {
            $nrEstruturaSup = $this->_determinaEstruturaSuperior(
                $hierarquiasTomador, $movimentacao->getNrestruturam(), $tpHierarquiaTomador->getIdautohierarq()
            );
            $this->_estruturaTomador->carregaEstrutura($nrEstruturaSup);
            return $this->_estruturaTomador->getEstrutura();
        }
    }
    
    public function retornaEstruturaResponsavel($nrVinculo, $dtInicio, $dtFim){

        try {
            $hierarquias = $this->_hierarquiaOrg->getItems();
            $tpHierarquia = $this->_tipoHierarquia->getItem(0);

            $this->_movimentacao->carregaMovimentacoesPeriodo($this->_nrOrg, $nrVinculo, $dtInicio, $dtFim);

            foreach ($this->_movimentacao->getItems() as $item) {
                if ((int)$item->getNrtipoestrutura() === (int)$tpHierarquia->getNrtipoestrutrel()) {
                    $nrEstruturaSup = $this->_determinaEstruturaSuperior(
                        $hierarquias, $item->getNrestruturam(), $tpHierarquia->getIdautohierarq()
                    );
                    if ($nrEstruturaSup === null) {
                        throw new \Exception('Estrutura ' . $item->getNrestruturam() . ' sem estrutura superior (hierarquia) definida.');
                    } else {
                        $this->_estrutura->carregaEstrutura($nrEstruturaSup);
                        $this->_movimentacaoVinculoResponsavel = $item;
                    }
                }
            }

            return $this->_estrutura->getItem(0);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaEstruturaTomador($nrVinculo, $dtInicio, $dtFim){
        $this->_estruturaTomador->resetItems();
        $this->_movimentacoesVinculoTomador = array();

        $hierarquiasTomador = $this->_hierarquiaOrgTomador->getItems();
        $tpHierarquiaTomador = $this->_tipoHierarquiaTomador->getItem(0);
        
        foreach($this->_movimentacao->getItems() as $item){
            if ((int)$item->getNrtipoestrutura() === (int)$tpHierarquiaTomador->getNrtipoestrutrel()) {
                $nrEstruturaSup = $this->_determinaEstruturaSuperior(
                    $hierarquiasTomador, $item->getNrestruturam(), $tpHierarquiaTomador->getIdautohierarq()
                );
                if ($nrEstruturaSup === null) {
                    MsgCalculo::getInstance()->adicionaMensagem('Verifique movimentação com estrutura nula para vinculo '.$nrVinculo);
                } else {
                    $this->_estruturaTomador->carregaEstrutura($nrEstruturaSup);
                    $this->_movimentacoesVinculoTomador[$nrEstruturaSup] = $item;
                }
            }
        }
        return $this->_estruturaTomador->getItems();
    }


    private function _determinaEstruturaSuperior(array $hierarquias, $nrEstrutura, $autoHierarquia){
        if ($autoHierarquia === TipohierarqorgEntity::IDAUTOHIERARQ_SIM) {
            $nrEstruturaSup = $nrEstrutura;
        } else {
            $nrEstruturaSup = $this->_hierarquiaOrg->determinaEstruturaSuperior($nrEstrutura, $hierarquias);
        }

        return $nrEstruturaSup;
    }

    public function determinaEnderecoEstrutura($estrutura){
        
        $nrParceiro = $estrutura->historico->getNrparcnegocio();
        if ($nrParceiro) {
            $endereco = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findOneBy(array('nrorg' => $this->_nrOrg,
            'nrparcnegocio' => $nrParceiro,'cdtipoendereco'=> self::ENDERECO_PRINCIPAL));
            
            if (!$endereco) {
                throw new \Exception('Conferir dados de endereço da estrutura '.$estrutura->historico->getNrestruturam().'.');
            }
            
            $municipio = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdpais' => $endereco->getCdpais(),
            'sgestado' => $endereco->getSgestado(),'cdmunicipio'=> $endereco->getCdmunicipio()));
            
            if (is_object($municipio)) {
                $endereco->municipio = $municipio->getNmmunicipio();
            } else {
                $endereco->municipio = null;
            }
            $endereco->bairro = $this->_bairro->defineBairro($endereco);
            if ($endereco->getDscompleendereco() !== null){
                $endereco->complemento = $endereco->getDscompleendereco();
            } else if(method_exists($endereco, 'getComplemento')) {
                $endereco->complemento = $endereco->getComplemento();
            } else {
                $endereco->complemento = null;
            }

            $objComunicacoesParc = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(array('nrorg' => $this->_nrOrg,
            'nrparcnegocio' => $estrutura->historico->getNrparcnegocio()));
            
            $endereco->telefone = null;
            $endereco->email = null;
            $endereco->ddd = null;
            $endereco->ramal = null;

            foreach ($objComunicacoesParc as $formaComunicacao){
                if ($formaComunicacao->getCdformacomu() ===  self::FORMA_COMUNICACAO_TEL_PRINCIPAL) {
                    $endereco->ddd = $formaComunicacao->getCdprefixcomuparc();
                    $endereco->telefone = $formaComunicacao->getCdcomunicaparc();
                    $endereco->ramal = '99999999';
                }
                if ($formaComunicacao->getCdformacomu() ===  self::FORMA_COMUNICACAO_EMAIL) {
                    $endereco->email = $formaComunicacao->getCdcomunicaparc();
                }

            }

            return $endereco;

        } else {
            return null;
        }
    }


    public function determinaAreaRisco($vinculo, $estrutura){
        $resultados = $this->_vinculoAreaRisco->retornaVinculoAreaRisco($this->_nrOrg, $vinculo->getNrvinculom());
        foreach ($resultados as $item) {
            $periodoValido = $this->_verificaPeriodoValidoMes($item->getDtfimvincarearisco());
            if ($periodoValido === true) {
                $areaRisco = $this->_areaRisco->retornaAreaRisco(
                    $this->_nrOrg, $this->_mesCompetencia, $item->getNrareariscom()
                );
            }
        }
        if (is_object($areaRisco) === false) {
            $resultados = $this->_estruturaAreaRisco->retornaEstruturaAreaRisco(
                $this->_nrOrg, $estrutura->getNrestruturam()
            );
            foreach ($resultados as $item) {
                $periodoValido = $this->_verificaPeriodoValidoMes($item->getDtfimestrutarearisco());
                if ($periodoValido === true) {
                    $areaRisco = $this->_areaRisco->retornaAreaRisco(
                        $this->_nrOrg, $this->_mesCompetencia, $item->getNrareariscom()
                    );
                }
            }
        }

        $this->_areaRisco = $areaRisco;
    }

    public function getDDD($sgestado)
    {
        return $this->_listaDDD[$sgestado];
    }

    public function getEstruturaRelacionada($nrVinculom)
    {
        $tpHierarquia = $this->_tipoHierarquia->getItem(0);
        $this->_movimentacao->carregaMovimentacoesPorTipoEstrutura($this->_nrOrg, $nrVinculom, $this->_ultimoDiaCompetencia,
            $tpHierarquia->getNrtipoestrutrel());

        foreach($this->_movimentacao->getItems() as $item){
            $this->_estrutura->carregaEstrutura($item->getNrestruturam());
            $estrutura = $this->_estrutura->getEstrutura();
            if($estrutura){
                return $estrutura;
            }
        }
        return null;
    }
}
