<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\IncidenciaIrffService;
use Zeedhi\Framework\DataSource\DataSet;

class IncidenciaIrrfController extends Crud {

    protected $dataSourceName_get = 'incidenciairrf_get';
    protected $dataSourceName_set = 'incidenciairrf_set';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, IncidenciaIrffService $incidenciaIrffService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->incidenciaIrffService = $incidenciaIrffService;
        parent::__construct($dataSourceManager);
    }

    public function getIncidenciaIrrf(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_get;
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if(isset($row['DSNRTIPOINCIDE'])){
                $row['DSNMTIPOINCIDE'] = explode(",", $row['DSNRTIPOINCIDE']);
                $row['DSNRTIPOINCIDE'] = explode(",", $row['DSNRTIPOINCIDE']);
                
                $length = count($row['DSNMTIPOINCIDE']);
                for ($i = 0; $i < $length; $i++) {
                	$incid = $this->incidenciaIrffService->retornaNomeIncidencia($row['DSNMTIPOINCIDE'][$i]);
                	$row['DSNMTIPOINCIDE'][$i] = $incid;
                }
                                
                foreach($row['DSNRTIPOINCIDE'] as $chave => $value)
                    $row['DSNRTIPOINCIDE'][$chave] = intval($value);
                    
                $row['NMTIPOINCIDE'] = explode(",", $row['NMTIPOINCIDE']);
            }    
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }
    
    public function setIncidenciaIrrf(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_set;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRINCIDIRRF', $row)) {
                    if ($row['NRINCIDIRRF'] === '') {
                        $row['NRINCIDIRRF'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'INCIDENCIAIRRF', 1, 12, true); 
                    }
                } else {
                    $row['NRINCIDIRRF'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'INCIDENCIAIRRF', 1, 12, true); 
                }

                $this->incidenciaIrffService->validaCamposVazios($row);
                
                if( isset($row['DSNRTIPOINCIDE']) && is_array($row['DSNRTIPOINCIDE']) )
                    $row['DSNRTIPOINCIDE'] = implode(",", $row['DSNRTIPOINCIDE']);
            
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteIncidenciaIrrf(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_set;
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 