<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\GpeReuniao;
use Folha\Entities\GpeReuniaoitem;
use Folha\Entities\GpeReuniaoparticip;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\DateUtil;

class ReuniaoService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function deleteReuniao($dataset) {
        $nrorg = $dataset["NRORG"];
        $nrreuniao = $dataset["NRREUNIAO"];
        $reuniaoitem = GpeReuniaoitem::findOneByNrreuniaoAndNrorg($nrreuniao, $nrorg);
        $participantes = GpeReuniaoparticip::findByNrreuniaoAndNrorg($nrreuniao, $nrorg);
        if (isset($participantes[0])) {
            throw new \Exception(Messages::HCM05000_SST_REUNIAO_EXISTS_PARTICIPANTES_ERROR, 10);
        }
        else if (isset($reuniaoitem)) {
            $idPauta = $reuniaoitem->getIdpautadeliberacao();
            if($idPauta === 'D'){
                throw new \Exception(Messages::HCM05000_SST_REUNIAO_EXISTS_DELIBERACAO_ERROR, 10);
            }else{
                throw new \Exception(Messages::HCM05000_SST_REUNIAO_EXISTS_PAUTA_ERROR, 10);
            }
        }
        else {
            $this->entityManager->beginTransaction();
            $reuniao = GpeReuniao::findOneByNrreuniaoAndNrorg($nrreuniao, $nrorg);
            $this->entityManager->remove($reuniao);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function cadastroAutomatico($nrreuniao,$participantes_list,$compareceu,$nrorg,$nrorgpadrao,$dtmescompetenc)
    {
        try{
            foreach($participantes_list as $participante){
                $participantes = GpeReuniaoparticip::findOneByNrreuniaoAndNrorgAndNrparcnegocio($nrreuniao, $nrorg, $participante);
                if(isset($participantes)){
                    continue;
                }
                $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAOPARTICIP', 1, 12, true);
                $obj = new \Folha\Entities\GpeReuniaoparticip();
                $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $obj->setDtinclusao(DateUtil::getDataAtual());
                $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $obj->setDtultatu(DateUtil::getDataAtual());
                $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $obj->setIdativo('S');
                $obj->setNrreuniao($nrreuniao);
                $obj->setNrreuniaoparticip($codigo);
                $obj->setNrparcnegocio($participante);
                $obj->setIdparticipcompareceu($compareceu);
                $obj->setDsobservacao(null);
                $this->entityManager->persist($obj);
                $this->entityManager->flush();
            }
        }catch(\Exception $e){
            throw $e;
        }
    }
}